use microrm::prelude::*;
use test_log::test;

mod common;

#[derive(Entity)]
struct Child {
    #[key]
    name: String,
    parent: ParentID,
}

#[derive(Entity)]
struct Parent {
    name: String,
}

#[derive(Database)]
struct PeopleDB {
    children: microrm::IDMap<Child>,
    parents: microrm::IDMap<Parent>,
}

#[test]
fn simple_query() {
    let db = common::open_test_db::<PeopleDB>("fkey_simple_query");

    let pid = db
        .parents
        .insert(Parent {
            name: "parent1".to_string(),
        })
        .unwrap();
    let _c1id = db
        .children
        .insert(Child {
            name: "child1".to_string(),
            parent: pid,
        })
        .unwrap();

    assert_eq!(
        db.children
            .foreign(Child::Parent)
            .get_ids()
            .expect("couldn't run query"),
        vec![pid]
    );

    assert_eq!(
        db.children
            .keyed("child1")
            .foreign(Child::Parent)
            .get_ids()
            .expect("couldn't run query"),
        Some(pid)
    );
}
